@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,7 +53,12 @@ import com.openexchange.config.ConfigurationService;
 
 /**
  * {@link ThreadPoolProperties} - Initialization of thread pool bundle.
- * 
+ *
+ * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+ */
+/**
+ * {@link ThreadPoolProperties}
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class ThreadPoolProperties {
@@ -72,6 +77,8 @@ public final class ThreadPoolProperties {
 
     private String refusedExecutionBehavior;
 
+    private boolean blocking;
+
     /**
      * Initializes a new {@link ThreadPoolProperties}.
      */
@@ -81,7 +88,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Initializes this properties with given configuration service.
-     * 
+     *
      * @param configurationService The configuration service to use
      * @return This instance with properties applied from configuration service
      */
@@ -159,6 +166,18 @@ public final class ThreadPoolProperties {
                 workQueue = "linked";
             }
 
+            blocking = false;
+            {
+                final String tmp = configurationService.getProperty("com.openexchange.threadpool.blocking");
+                if (null != tmp) {
+                    try {
+                        blocking = Boolean.parseBoolean(tmp.trim());
+                    } catch (final NumberFormatException e) {
+                        blocking = false;
+                    }
+                }
+            }
+
             refusedExecutionBehavior = "abort";
             {
                 final String tmp = configurationService.getProperty("com.openexchange.threadpool.refusedExecutionBehavior");
@@ -174,7 +193,7 @@ public final class ThreadPoolProperties {
             workQueue = "synchronous";
             refusedExecutionBehavior = "abort";
         }
-        final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(ThreadPoolProperties.class);
+        final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(ThreadPoolProperties.class));
         if (LOG.isInfoEnabled()) {
             final StringBuilder sb = new StringBuilder(512);
             sb.append("Thread Pool Configuration:\n");
@@ -190,7 +209,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Gets the corePoolSize
-     * 
+     *
      * @return The corePoolSize
      */
     public int getCorePoolSize() {
@@ -199,7 +218,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Gets the prestartAllCoreThreads
-     * 
+     *
      * @return The prestartAllCoreThreads
      */
     public boolean isPrestartAllCoreThreads() {
@@ -208,7 +227,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Gets the maximumPoolSize
-     * 
+     *
      * @return The maximumPoolSize
      */
     public int getMaximumPoolSize() {
@@ -217,7 +236,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Gets the keepAliveTime
-     * 
+     *
      * @return The keepAliveTime
      */
     public long getKeepAliveTime() {
@@ -226,7 +245,7 @@ public final class ThreadPoolProperties {
 
     /**
      * Gets the workQueue
-     * 
+     *
      * @return The workQueue
      */
     public String getWorkQueue() {
@@ -238,16 +257,25 @@ public final class ThreadPoolProperties {
      * <p>
      * Note: If size of work queue is set to a value greater than zero, {@link #getWorkQueue()} is implicitly set to <code>"linked"</code>
      * to accomplish a bounded work queue.
-     * 
+     *
      * @return The work queue size
      */
     public int getWorkQueueSize() {
         return workQueueSize;
     }
 
+    /**
+     * Whether a blocking behavior should be applied.
+     *
+     * @return <code>true</code> if a blocking behavior should be applied; otherwsie <code>false</code>
+     */
+    public boolean isBlocking() {
+        return blocking;
+    }
+
     /**
      * Gets the refusedExecutionBehavior
-     * 
+     *
      * @return The refusedExecutionBehavior
      */
     public String getRefusedExecutionBehavior() {
