@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,6 +60,7 @@ import java.util.concurrent.CompletionService;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorCompletionService;
+import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Future;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.ThreadPoolExecutor;
@@ -73,16 +74,16 @@ import com.openexchange.threadpool.ThreadPoolService;
 /**
  * {@link ThreadPoolServiceImpl} - A thread pool backed by a {@link ThreadPoolExecutor} instance which is accessible via
  * {@link #getExecutor()}.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class ThreadPoolServiceImpl implements ThreadPoolService {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(ThreadPoolServiceImpl.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(ThreadPoolServiceImpl.class));
 
     /**
      * Creates a new {@link ThreadPoolServiceImpl} with the given properties.
-     * 
+     *
      * @param properties The properties
      * @return A new {@link ThreadPoolServiceImpl} instance
      */
@@ -93,18 +94,20 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
             properties.getKeepAliveTime(),
             properties.getWorkQueue(),
             properties.getWorkQueueSize(),
+            properties.isBlocking(),
             properties.getRefusedExecutionBehavior());
     }
 
     /**
      * Creates a new {@link ThreadPoolServiceImpl} with the given initial parameters.
-     * 
+     *
      * @param corePoolSize The number of threads to keep in the pool, even if they are idle.
      * @param maximumPoolSize The maximum number of threads to allow in the pool.
      * @param keepAliveTime When the number of threads is greater than the core, this is the maximum time in milliseconds that excess idle
      *            threads will wait for new tasks before terminating.
      * @param workQueue The queue to use for holding tasks before they are executed.
      * @param workQueueSize The size of the work queue; zero for unlimited size
+     * @param blocking <code>true</code> for a blocking behavior; otherwise <code>false</code>
      * @param refusedExecutionBehavior The default behavior to obey when execution is blocked because the thread bounds and queue capacities
      *            are reached.
      * @return A new {@link ThreadPoolServiceImpl} instance
@@ -113,23 +116,21 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
      *             be resolved.
      * @throws NullPointerException If <tt>workQueue</tt> or <tt>refusedExecutionBehavior</tt> are <code>null</code>.
      */
-    public static ThreadPoolServiceImpl newInstance(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final String workQueue, final int workQueueSize, final String refusedExecutionBehavior) {
+    public static ThreadPoolServiceImpl newInstance(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final String workQueue, final int workQueueSize, final boolean blocking, final String refusedExecutionBehavior) {
         final RejectedExecutionType ret = RejectedExecutionType.getRejectedExecutionType(refusedExecutionBehavior);
         if (null == ret) {
             throw new IllegalArgumentException("Unknown refused execution behavior: " + refusedExecutionBehavior);
         }
-        final ThreadPoolServiceImpl newInst =
-            new ThreadPoolServiceImpl(corePoolSize, maximumPoolSize, keepAliveTime, workQueue, workQueueSize);
-        final DelegatingRejectedExecutionHandler reh = new DelegatingRejectedExecutionHandler(ret.getHandler(), newInst);
-        newInst.threadPoolExecutor.setRejectedExecutionHandler(reh);
-        return newInst;
+        return new ThreadPoolServiceImpl(corePoolSize, maximumPoolSize, keepAliveTime, workQueue, workQueueSize, blocking, refusedExecutionBehavior);
     }
 
     private final CustomThreadPoolExecutor threadPoolExecutor;
 
+    private final int corePoolSize;
+
     /**
-     * Creates a new <tt>ThreadPoolExecutor</tt> with the given initial parameters.
-     * 
+     * Creates a new <tt>ThreadPoolServiceImpl</tt> with the given initial parameters.
+     *
      * @param corePoolSize The number of threads to keep in the pool, even if they are idle.
      * @param maximumPoolSize The maximum number of threads to allow in the pool.
      * @param keepAliveTime When the number of threads is greater than the core, this is the maximum time in milliseconds that excess idle
@@ -140,19 +141,44 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
      *             or if corePoolSize greater than maximumPoolSize.
      * @throws NullPointerException if <tt>workQueue</tt> or <tt>threadFactory</tt> are <code>null</code>.
      */
-    private ThreadPoolServiceImpl(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final String workQueue, final int workQueueSize) {
+    private ThreadPoolServiceImpl(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final String workQueue, final int workQueueSize, final boolean blocking, final String refusedExecutionBehavior) {
         final QueueType queueType = QueueType.getQueueType(workQueue);
         if (null == queueType) {
             throw new IllegalArgumentException("Unknown queue type: " + workQueue);
         }
-        threadPoolExecutor =
-            new CustomThreadPoolExecutor(
-                getCorePoolSize(corePoolSize),
-                maximumPoolSize,
-                keepAliveTime,
-                TimeUnit.MILLISECONDS,
-                queueType.newWorkQueue(workQueueSize),
-                new CustomThreadFactory("OXWorker-"));
+        final RejectedExecutionType ret = RejectedExecutionType.getRejectedExecutionType(refusedExecutionBehavior);
+        if (null == ret) {
+            throw new IllegalArgumentException("Unknown refused execution behavior: " + refusedExecutionBehavior);
+        }
+        this.corePoolSize = getCorePoolSize(corePoolSize);
+        if (QueueType.LINKED.equals(queueType) && corePoolSize < maximumPoolSize) {
+            final ScalingQueue scalingQueue = workQueueSize > 0 ? new ScalingQueue(workQueueSize) : new ScalingQueue();
+            threadPoolExecutor =
+                new CustomThreadPoolExecutor(
+                    this.corePoolSize,
+                    maximumPoolSize,
+                    keepAliveTime,
+                    TimeUnit.MILLISECONDS,
+                    scalingQueue,
+                    new CustomThreadFactory("OXWorker-"));
+            scalingQueue.setThreadPoolExecutor(threadPoolExecutor);
+            scalingQueue.setThreadPool(this);
+            final DelegatingRejectedExecutionHandler reh = new DelegatingRejectedExecutionHandler(ret.getHandler(), this);
+            threadPoolExecutor.setRejectedExecutionHandler(scalingQueue.createRejectedExecutionHandler(reh));
+            threadPoolExecutor.setBlocking(false);
+        } else {
+            threadPoolExecutor =
+                new CustomThreadPoolExecutor(
+                    queueType.isFixedSize() ? maximumPoolSize : this.corePoolSize,
+                    maximumPoolSize,
+                    keepAliveTime,
+                    TimeUnit.MILLISECONDS,
+                    queueType.newWorkQueue(workQueueSize),
+                    new CustomThreadFactory("OXWorker-"));
+            final DelegatingRejectedExecutionHandler reh = new DelegatingRejectedExecutionHandler(ret.getHandler(), this);
+            threadPoolExecutor.setRejectedExecutionHandler(reh);
+            threadPoolExecutor.setBlocking(blocking);
+        }
     }
 
     private static int getCorePoolSize(final int desiredCorePoolSize) {
@@ -170,7 +196,7 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
     /**
      * Blocks until all tasks have completed execution after a shutdown request, or the timeout occurs, or the current thread is
      * interrupted, whichever happens first.
-     * 
+     *
      * @param timeout The maximum time in milliseconds to wait
      * @return <code>true</code> if this thread pool terminated and <code>false</code> if the timeout elapsed before termination
      * @throws InterruptedException If interrupted while waiting
@@ -202,7 +228,7 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
      * <p>
      * This implementation cancels tasks via Thread.interrupt, so if any tasks mask or fail to respond to interrupts, they may never
      * terminate.
-     * 
+     *
      * @return A list of tasks that never commenced execution
      */
     public List<Runnable> shutdownNow() {
@@ -331,7 +357,7 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
         return new CompletionFutureImpl<T>(completionService);
     }
 
-    public <T> CompletionFuture<T> invoke(Task<T>[] tasks) {
+    public <T> CompletionFuture<T> invoke(final Task<T>[] tasks) {
         if (tasks == null) {
             throw new NullPointerException();
         }
@@ -353,10 +379,27 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
         return new CompletionFutureImpl<T>(completionService);
     }
 
-    public Executor getExecutor() {
+    /**
+     * Gets the thread pool executor.
+     *
+     * @return The thread pool executor
+     */
+    public CustomThreadPoolExecutor getThreadPoolExecutor() {
         return threadPoolExecutor;
     }
 
+    public ExecutorService getExecutor() {
+        return new DelegateExecutorService(threadPoolExecutor);
+    }
+
+    public ExecutorService getFixedExecutor() {
+        return new FixedExecutorService(corePoolSize, threadPoolExecutor);
+    }
+
+    public ExecutorService getFixedExecutor(final int size) {
+        return new FixedExecutorService(size, threadPoolExecutor);
+    }
+
     public boolean isShutdown() {
         return threadPoolExecutor.isShutdown();
     }
@@ -413,7 +456,7 @@ public final class ThreadPoolServiceImpl implements ThreadPoolService {
         /**
          * Creates an {@link CustomExecutorCompletionService} using the supplied executor for base task execution and a
          * {@link LinkedBlockingQueue} as a completion queue.
-         * 
+         *
          * @param executor the executor to use
          * @throws NullPointerException if executor is <tt>null</tt>
          */
